'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var navigation = require('navigation');
var React = require('react');
var React__default = _interopDefault(React);
var navigationReact = require('navigation-react');

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
    return extendStatics(d, b);
};
function __extends(d, b) {
    if (typeof b !== "function" && b !== null)
        throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}
var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}
function __spreadArrays() {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
}

var MobileHistoryManager = /** @class */ (function (_super) {
    __extends(MobileHistoryManager, _super);
    function MobileHistoryManager(buildCurrentUrl, applicationPath) {
        var _this = _super.call(this, applicationPath || '') || this;
        _this.navigateHistory = null;
        _this.backCrumb = null;
        _this.hash = false;
        _this.buildCurrentUrl = buildCurrentUrl;
        _this.hash = applicationPath === undefined;
        return _this;
    }
    MobileHistoryManager.prototype.init = function (navigateHistory, rewriteUrl) {
        var _this = this;
        if (!this.rewriteUrl)
            this.rewriteUrl = rewriteUrl;
        if (!this.disabled && !this.navigateHistory) {
            this.navigateHistory = function (e) {
                var _a;
                var link = (_a = e.state) === null || _a === void 0 ? void 0 : _a.navigationLink;
                if (link && _this.backCrumb !== null) {
                    var distance = _this.backCrumb - link.split('crumb=').length + 1;
                    if (distance < 0)
                        window.history.go(distance);
                    else
                        _this.backCrumb = null;
                }
                else {
                    _this.backCrumb = null;
                    navigateHistory(link);
                }
            };
            window.addEventListener('popstate', this.navigateHistory);
        }
    };
    MobileHistoryManager.prototype.addHistory = function (url, replace, stateContext) {
        this.backCrumb = null;
        var title = typeof document !== 'undefined' && document.title;
        var distance = 0;
        if (!!stateContext && !stateContext.history) {
            var oldUrl = stateContext.oldUrl, crumbs = stateContext.crumbs;
            var start = !oldUrl ? 0 : oldUrl.split('crumb=').length;
            distance = crumbs.length - start + 1;
            for (var i = start; i < crumbs.length; i++) {
                var _a = crumbs[i], url_1 = _a.url, state = _a.state;
                _super.prototype.addHistory.call(this, url_1, i === 0);
                if (typeof document !== 'undefined' && state.title)
                    document.title = state.title;
            }
            if (!this.disabled && distance < 0) {
                this.backCrumb = crumbs.length;
                window.history.go(distance);
            }
        }
        if (this.backCrumb === null)
            _super.prototype.addHistory.call(this, url, replace);
        if (title)
            document.title = title;
    };
    MobileHistoryManager.prototype.getHref = function (url) {
        var _a;
        url = ((_a = this.rewriteUrl) === null || _a === void 0 ? void 0 : _a.call(this, url)) || url;
        var hashIndex = url.indexOf('#');
        var pathAndQuery = hashIndex < 0 ? url : url.substring(0, hashIndex);
        var queryIndex = pathAndQuery.indexOf('?');
        if (queryIndex >= 0) {
            var path = pathAndQuery.substring(0, queryIndex);
            var query = pathAndQuery.substring(queryIndex + 1);
            var params = query.split('&');
            var crumblessParams = [];
            for (var i = 0; i < params.length; i++) {
                if (params[i].substring(0, 6) !== 'crumb=') {
                    crumblessParams.push(params[i]);
                }
            }
            var crumblessQuery = crumblessParams.join('&');
            var hash = hashIndex >= 0 ? url.substring(hashIndex) : '';
            url = "" + path + (crumblessQuery && '?') + crumblessQuery + hash;
        }
        return !this.hash ? _super.prototype.getHref.call(this, url) : '#' + url;
    };
    MobileHistoryManager.prototype.getUrl = function (hrefElement) {
        return !this.hash ? _super.prototype.getUrl.call(this, hrefElement) : hrefElement.hash.substring(1);
    };
    MobileHistoryManager.prototype.getCurrentUrl = function () {
        var url = _super.prototype.getCurrentUrl.call(this);
        if (this.buildCurrentUrl)
            url = this.buildCurrentUrl(url);
        return url;
    };
    MobileHistoryManager.prototype.stop = function () {
        if (this.navigateHistory)
            window.removeEventListener('popstate', this.navigateHistory);
        this.navigateHistory = null;
    };
    return MobileHistoryManager;
}(navigation.HTML5HistoryManager));

var Motion = /** @class */ (function (_super) {
    __extends(Motion, _super);
    function Motion(props) {
        var _this = _super.call(this, props) || this;
        _this.animate = _this.animate.bind(_this);
        _this.state = { items: [], restart: false };
        return _this;
    }
    Motion.getDerivedStateFromProps = function (props, _a) {
        var prevItems = _a.items;
        var tick = typeof window !== 'undefined' && window.performance ? window.performance.now() : 0;
        var _b = Motion.move(tick, prevItems, props), items = _b.items, moving = _b.moving;
        return { items: items, restart: moving };
    };
    Motion.prototype.componentDidMount = function () {
        this.animateId = window.requestAnimationFrame(this.animate);
    };
    Motion.prototype.componentDidUpdate = function () {
        if (!this.animateId && this.state.restart)
            this.animateId = window.requestAnimationFrame(this.animate);
    };
    Motion.prototype.componentWillUnmount = function () {
        window.cancelAnimationFrame(this.animateId);
    };
    Motion.prototype.animate = function (tick) {
        var _this = this;
        this.setState(function (_a) {
            var prevItems = _a.items;
            var items = Motion.move(tick, prevItems, _this.props).items;
            _this.animateId = null;
            return { items: items, restart: false };
        });
    };
    Motion.move = function (tick, prevItems, props) {
        var data = props.data, enter = props.enter, leave = props.leave, update = props.update, progress = props.progress, getKey = props.getKey, duration = props.duration, onRest = props.onRest;
        var dataByKey = data.reduce(function (acc, item, index) {
            var _a;
            return (__assign(__assign({}, acc), (_a = {}, _a[getKey(item)] = __assign(__assign({}, item), { index: index }), _a)));
        }, {});
        var itemsByKey = prevItems.reduce(function (acc, item) {
            var _a;
            return (__assign(__assign({}, acc), (_a = {}, _a[item.key] = item, _a)));
        }, {});
        var items = prevItems
            .map(function (item) {
            var _a;
            var matchedItem = dataByKey[item.key];
            var nextItem = { key: item.key, data: matchedItem || item.data, tick: tick };
            nextItem.end = !matchedItem ? (leave || update)(item.data) : update(matchedItem);
            nextItem.index = !matchedItem ? item.index : matchedItem.index;
            var unchanged = Motion.areEqual(item.end, nextItem.end);
            if (unchanged) {
                nextItem.start = item.start;
                nextItem.rest = item.progress === 1;
                var itemDuration = (_a = nextItem.end.duration) !== null && _a !== void 0 ? _a : duration;
                var progressDelta = itemDuration > 0 ? Math.min(nextItem.tick - item.tick, 50) / itemDuration : 1;
                nextItem.progress = Math.min(item.progress + progressDelta, 1);
            }
            else {
                nextItem.rest = false;
                var reverse = !unchanged && Motion.areEqual(item.start, nextItem.end);
                nextItem.start = reverse ? item.end : (!progress ? item.style : item.start);
                nextItem.progress = reverse ? 1 - item.progress : progress;
            }
            nextItem.style = Motion.interpolateStyle(nextItem);
            if (onRest && nextItem.rest && !item.rest)
                onRest(item.data);
            return nextItem;
        })
            .filter(function (item) { return dataByKey[item.key] || (!item.rest && leave); })
            .concat(data
            .filter(function (item) { return !itemsByKey[getKey(item)]; })
            .map(function (item) {
            var index = dataByKey[getKey(item)].index;
            var newItem = { key: getKey(item), data: item, tick: tick, rest: false, index: index };
            newItem.start = newItem.style = enter(item);
            newItem.end = update(item);
            newItem.progress = Motion.areEqual(newItem.start, newItem.end) ? 1 : progress;
            return newItem;
        }))
            .sort(function (a, b) { return a.index !== b.index ? a.index - b.index : a.key.length - b.key.length; });
        return { items: items, moving: items.filter(function (_a) {
                var rest = _a.rest;
                return !rest;
            }).length !== 0 };
    };
    Motion.areEqual = function (_a, _b) {
        if (_a === void 0) { _a = {}; }
        if (_b === void 0) { _b = {}; }
        var _c = _a.duration, from = __rest(_a, ["duration"]);
        var _d = _b.duration, to = __rest(_b, ["duration"]);
        if (Object.keys(from).length !== Object.keys(to).length)
            return false;
        for (var key in from) {
            if (from[key] !== to[key])
                return false;
        }
        return true;
    };
    Motion.interpolateStyle = function (_a) {
        var start = _a.start, end = _a.end, progress = _a.progress;
        var style = {};
        for (var key in end) {
            if (key !== 'duration')
                style[key] = start[key] + (progress * (end[key] - start[key]));
        }
        style.duration = end.duration;
        return style;
    };
    Motion.prototype.render = function () {
        return this.props.children(this.state.items);
    };
    Motion.defaultProps = {
        progress: 0,
    };
    return Motion;
}(React.Component));

var withStateNavigator = (function (Component) { return function (props) { return (React.createElement(navigationReact.NavigationContext.Consumer, null, function (navigationEvent) { return React.createElement(Component, __assign({ stateNavigator: navigationEvent.stateNavigator, navigationEvent: navigationEvent }, props)); })); }; });

var Scene = /** @class */ (function (_super) {
    __extends(Scene, _super);
    function Scene(props) {
        var _this = _super.call(this, props) || this;
        _this.state = { navigationEvent: null };
        return _this;
    }
    Scene.getDerivedStateFromProps = function (props, _a) {
        var prevNavigationEvent = _a.navigationEvent;
        var crumb = props.crumb, navigationEvent = props.navigationEvent;
        var _b = navigationEvent.stateNavigator.stateContext, state = _b.state, oldState = _b.oldState, oldUrl = _b.oldUrl, crumbs = _b.crumbs;
        if (!state || crumbs.length !== crumb)
            return null;
        if (!oldUrl || !prevNavigationEvent)
            return { navigationEvent: navigationEvent };
        var oldCrumbs = navigationEvent.stateNavigator.parseLink(oldUrl).crumbs;
        var replace = oldCrumbs.length === crumb && oldState !== state;
        return !replace ? { navigationEvent: navigationEvent } : null;
    };
    Scene.prototype.shouldComponentUpdate = function (_a, nextState) {
        var crumb = _a.crumb, rest = _a.rest, stateNavigator = _a.navigationEvent.stateNavigator;
        var crumbs = stateNavigator.stateContext.crumbs;
        var freezableOrCurrent = rest && (!!React.Suspense || crumbs.length === crumb);
        return freezableOrCurrent || nextState.navigationEvent !== this.state.navigationEvent;
    };
    Scene.prototype.render = function () {
        var _a;
        var navigationEvent = this.state.navigationEvent;
        var _b = this.props, crumb = _b.crumb, stateNavigator = _b.navigationEvent.stateNavigator;
        var crumbs = stateNavigator.stateContext.crumbs;
        var stateContext = (_a = navigationEvent === null || navigationEvent === void 0 ? void 0 : navigationEvent.stateNavigator) === null || _a === void 0 ? void 0 : _a.stateContext;
        var _c = stateContext || crumbs[crumb] || {}, state = _c.state, data = _c.data;
        return (React.createElement(navigationReact.NavigationContext.Provider, { value: navigationEvent }, navigationEvent && this.props.renderScene(state, data)));
    };
    Scene.defaultProps = {
        renderScene: function (state, data) { return state.renderScene(data); }
    };
    return Scene;
}(React.Component));
var Scene$1 = withStateNavigator(Scene);

var SharedElementRegistry = /** @class */ (function () {
    function SharedElementRegistry() {
        this.sharedElements = {};
    }
    SharedElementRegistry.prototype.registerSharedElement = function (scene, name, ref, data) {
        this.sharedElements[scene] = this.sharedElements[scene] || {};
        this.sharedElements[scene][name] = { ref: ref, data: data };
    };
    SharedElementRegistry.prototype.unregisterSharedElement = function (scene, name) {
        if (this.sharedElements[scene]) {
            if (name)
                delete this.sharedElements[scene][name];
            else
                delete this.sharedElements[scene];
        }
    };
    SharedElementRegistry.prototype.getSharedElements = function (scene, oldScene) {
        if (scene === oldScene)
            return [];
        var oldSharedElements = this.sharedElements[oldScene];
        var mountedSharedElements = this.sharedElements[scene];
        var sharedElements = [];
        for (var name in mountedSharedElements) {
            if (oldSharedElements && oldSharedElements[name]) {
                sharedElements.push({
                    name: name,
                    oldElement: oldSharedElements[name],
                    mountedElement: mountedSharedElements[name]
                });
            }
        }
        return sharedElements;
    };
    return SharedElementRegistry;
}());

var SharedElementContext = React.createContext(new SharedElementRegistry());

/**
 * The MIT License (MIT)
 *
 * Copyright (c) 2021 Software Mansion
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
var Suspender = function (_a) {
    var freeze = _a.freeze, children = _a.children;
    var promiseCache = React.useRef ? React.useRef({}).current : {};
    if (freeze && !promiseCache.promise) {
        promiseCache.promise = new Promise(function (resolve) {
            promiseCache.resolve = resolve;
        });
        throw promiseCache.promise;
    }
    else if (freeze) {
        throw promiseCache.promise;
    }
    else if (promiseCache.promise) {
        promiseCache.resolve();
        promiseCache.promise = undefined;
    }
    return children;
};
var Freeze = function (_a) {
    var enabled = _a.enabled, children = _a.children;
    var suspendable = !!React__default.Suspense && typeof window !== 'undefined';
    var suspender = React__default.createElement(Suspender, { freeze: enabled && suspendable }, children);
    return suspendable ? React__default.createElement(React.Suspense, { fallback: null }, suspender) : suspender;
};

var NavigationMotion = function (_a) {
    var unmountedStyleStack = _a.unmountedStyle, mountedStyleStack = _a.mountedStyle, crumbStyleStack = _a.crumbStyle, _b = _a.duration, duration = _b === void 0 ? 300 : _b, sharedElementMotion = _a.sharedElementMotion, renderScene = _a.renderScene, children = _a.children, stackInvalidatedLink = _a.stackInvalidatedLink, _c = _a.renderMotion, renderMotion = _c === void 0 ? children : _c;
    var sharedElementRegistry = React.useRef(new SharedElementRegistry());
    var stateNavigator = React.useContext(navigationReact.NavigationContext).stateNavigator;
    var _d = React.useState({ stateNavigator: null, keys: [] }), motionState = _d[0], setMotionState = _d[1];
    var scenes = {};
    var firstLink;
    var findScenes = function (elements, nested) {
        if (elements === void 0) { elements = children; }
        if (nested === void 0) { nested = false; }
        for (var _i = 0, _a = React__default.Children.toArray(elements); _i < _a.length; _i++) {
            var scene = _a[_i];
            var _b = scene.props, stateKey = _b.stateKey, children_1 = _b.children;
            if (scene.type === NavigationMotion.Scene) {
                firstLink = firstLink || stateNavigator.fluent().navigate(stateKey).url;
                scenes[stateKey] = scene;
            }
            else if (!nested)
                findScenes(children_1, true);
        }
    };
    findScenes();
    var prevScenes = React.useRef({});
    var allScenes = __assign(__assign({}, prevScenes.current), scenes);
    React.useEffect(function () {
        prevScenes.current = allScenes;
        var _a = stateNavigator.stateContext, state = _a.state, crumbs = _a.crumbs, nextCrumb = _a.nextCrumb;
        var validate = function (_a) {
            var key = _a.key;
            return !!scenes[key];
        };
        if (firstLink) {
            stateNavigator.onBeforeNavigate(validate);
            var resetLink = !state ? firstLink : undefined;
            if (!resetLink && __spreadArrays(crumbs, [nextCrumb]).find(function (_a) {
                var state = _a.state;
                return !scenes[state.key];
            }))
                resetLink = stackInvalidatedLink != null ? stackInvalidatedLink : firstLink;
            if (resetLink != null)
                stateNavigator.navigateLink(resetLink);
        }
        return function () { return stateNavigator.offBeforeNavigate(validate); };
    }, [children, stateNavigator, scenes, allScenes, stackInvalidatedLink]);
    var getSharedElements = function () {
        var _a = stateNavigator.stateContext, crumbs = _a.crumbs, oldUrl = _a.oldUrl;
        if (oldUrl !== null) {
            var oldScene = oldUrl.split('crumb=').length - 1;
            return sharedElementRegistry.current.getSharedElements(crumbs.length, oldScene);
        }
        return [];
    };
    var clearScene = function (index) {
        var scene = getScenes().filter(function (scene) { return scene.key === index; })[0];
        if (!scene)
            sharedElementRegistry.current.unregisterSharedElement(index);
    };
    var getScenes = function () {
        var keys = motionState.keys;
        var _a = stateNavigator.stateContext, crumbs = _a.crumbs, nextCrumb = _a.nextCrumb, oldUrl = _a.oldUrl;
        var backward = oldUrl && oldUrl.split('crumb=').length > crumbs.length + 1;
        return crumbs.concat(nextCrumb).map(function (_a, index, crumbsAndNext) {
            var state = _a.state, data = _a.data, url = _a.url;
            var preCrumbs = crumbsAndNext.slice(0, index);
            var _b = crumbsAndNext[index + 1] || { state: undefined, data: undefined }, nextState = _b.state, nextData = _b.data;
            var mount = url === nextCrumb.url;
            var fromUnmounted = mount && !backward;
            return { key: keys[index], state: state, data: data, url: url, crumbs: preCrumbs, nextState: nextState, nextData: nextData, mount: mount, fromUnmounted: fromUnmounted };
        });
    };
    var sceneProps = function (_a) {
        var key = _a.key;
        return firstLink ? allScenes[key].props : null;
    };
    var returnOrCall = function (item) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        return typeof item !== 'function' ? item : item.apply(void 0, args);
    };
    var unmountedStyle = function (state) {
        var _a, _b;
        var rest = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            rest[_i - 1] = arguments[_i];
        }
        return ((_a = sceneProps(state)) === null || _a === void 0 ? void 0 : _a.unmountedStyle) ? returnOrCall.apply(void 0, __spreadArrays([(_b = sceneProps(state)) === null || _b === void 0 ? void 0 : _b.unmountedStyle], rest)) : returnOrCall.apply(void 0, __spreadArrays([unmountedStyleStack, state], rest));
    };
    var mountedStyle = function (state) {
        var _a, _b;
        var rest = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            rest[_i - 1] = arguments[_i];
        }
        return ((_a = sceneProps(state)) === null || _a === void 0 ? void 0 : _a.mountedStyle) ? returnOrCall.apply(void 0, __spreadArrays([(_b = sceneProps(state)) === null || _b === void 0 ? void 0 : _b.mountedStyle], rest)) : returnOrCall.apply(void 0, __spreadArrays([mountedStyleStack, state], rest));
    };
    var crumbStyle = function (state) {
        var _a, _b;
        var rest = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            rest[_i - 1] = arguments[_i];
        }
        return ((_a = sceneProps(state)) === null || _a === void 0 ? void 0 : _a.crumbStyle) ? returnOrCall.apply(void 0, __spreadArrays([(_b = sceneProps(state)) === null || _b === void 0 ? void 0 : _b.crumbStyle], rest)) : returnOrCall.apply(void 0, __spreadArrays([crumbStyleStack, state], rest));
    };
    var getStyle = function (mounted, _a) {
        var state = _a.state, data = _a.data, crumbs = _a.crumbs, nextState = _a.nextState, nextData = _a.nextData, mount = _a.mount, fromUnmounted = _a.fromUnmounted;
        var styleProp = !mounted ? unmountedStyle : (mount ? mountedStyle : crumbStyle);
        fromUnmounted = (mounted && mount) ? fromUnmounted : undefined;
        return __assign({}, styleProp(state, data, crumbs, nextState, nextData, fromUnmounted));
    };
    var getMotion = function (styles) {
        var moving = false, mountMoving = false, mountDuration, mountProgress;
        for (var _i = 0, styles_1 = styles; _i < styles_1.length; _i++) {
            var _a = styles_1[_i], rest = _a.rest, progress = _a.progress, mount = _a.data.mount, duration_1 = _a.style.duration;
            if (mount) {
                mountMoving = !rest;
                mountDuration = duration_1;
                mountProgress = progress;
            }
            moving = moving || !rest;
        }
        return { rest: !moving, mountRest: !mountMoving, mountDuration: mountDuration, mountProgress: mountProgress };
    };
    var prevStateNavigator = motionState.stateNavigator;
    if (prevStateNavigator !== stateNavigator && stateNavigator.stateContext.state) {
        setMotionState(function (prevStackState) {
            var prevKeys = prevStackState.keys, prevStateNavigator = prevStackState.stateNavigator;
            var _a = stateNavigator.stateContext, state = _a.state, crumbs = _a.crumbs, nextCrumb = _a.nextCrumb;
            var prevState = prevStateNavigator && prevStateNavigator.stateContext.state;
            var currentKeys = crumbs.concat(nextCrumb).map(function (_, i) { return '' + i; });
            var newKeys = currentKeys.slice(prevKeys.length);
            var keys = prevKeys.slice(0, currentKeys.length).concat(newKeys);
            if (prevKeys.length === keys.length && prevState !== state)
                keys[keys.length - 1] += '+';
            return { keys: keys, stateNavigator: stateNavigator };
        });
    }
    var _e = stateNavigator.stateContext, crumbs = _e.crumbs, oldState = _e.oldState, stateContext = stateNavigator.stateContext;
    renderScene = firstLink ? function (_a) {
        var key = _a.key;
        return allScenes[key];
    } : renderScene;
    return (stateContext.state &&
        React__default.createElement(SharedElementContext.Provider, { value: sharedElementRegistry.current },
            React__default.createElement(Motion, { data: getScenes(), getKey: function (_a) {
                    var key = _a.key;
                    return key;
                }, enter: function (scene) { return getStyle(!oldState, scene); }, update: function (scene) { return getStyle(true, scene); }, leave: function (scene) { return getStyle(false, scene); }, onRest: function (_a) {
                    var key = _a.key;
                    return clearScene(key);
                }, duration: duration }, function (styles) {
                var _a = getMotion(styles), rest = _a.rest, mountRest = _a.mountRest, mountDuration = _a.mountDuration, mountProgress = _a.mountProgress;
                return (styles.map(function (_a) {
                    var _b = _a.data, key = _b.key, state = _b.state, data = _b.data, _c = _a.style, duration = _c.duration, style = __rest(_c, ["duration"]);
                    var crumb = +key.replace(/\++$/, '');
                    var scene = React__default.createElement(Scene$1, { crumb: crumb, rest: rest, renderScene: renderScene });
                    return (React__default.createElement(Freeze, { key: key, enabled: rest && crumb < getScenes().length - 1 }, renderMotion(style, scene, key, crumbs.length === crumb, state, data)));
                }).concat(sharedElementMotion && sharedElementMotion({
                    key: 'sharedElements',
                    sharedElements: !mountRest ? getSharedElements() : [],
                    progress: mountProgress,
                    duration: mountDuration !== null && mountDuration !== void 0 ? mountDuration : duration,
                })));
            })));
};
NavigationMotion.Scene = function (_a) {
    var children = _a.children;
    return children;
};

var SharedElement = /** @class */ (function (_super) {
    __extends(SharedElement, _super);
    function SharedElement(props) {
        var _this = _super.call(this, props) || this;
        _this.ref = React.createRef();
        return _this;
    }
    SharedElement.prototype.componentDidMount = function () {
        this.register();
    };
    SharedElement.prototype.componentDidUpdate = function (prevProps) {
        var _a = this.props, stateNavigator = _a.stateNavigator, sharedElementRegistry = _a.sharedElementRegistry;
        var scene = stateNavigator.stateContext.crumbs.length;
        sharedElementRegistry.unregisterSharedElement(scene, prevProps.name);
        this.register();
    };
    SharedElement.prototype.componentWillUnmount = function () {
        var _a = this.props, stateNavigator = _a.stateNavigator, sharedElementRegistry = _a.sharedElementRegistry;
        var scene = stateNavigator.stateContext.crumbs.length;
        sharedElementRegistry.unregisterSharedElement(scene, this.props.name);
    };
    SharedElement.prototype.register = function () {
        var _a = this.props, unshare = _a.unshare, name = _a.name, data = _a.data, stateNavigator = _a.stateNavigator, sharedElementRegistry = _a.sharedElementRegistry;
        var scene = stateNavigator.stateContext.crumbs.length;
        if (!unshare)
            sharedElementRegistry.registerSharedElement(scene, name, this.ref.current, data);
        else
            sharedElementRegistry.unregisterSharedElement(scene, name);
    };
    SharedElement.prototype.render = function () {
        return React.cloneElement(this.props.children, { ref: this.ref });
    };
    return SharedElement;
}(React.Component));
var SharedElement$1 = withStateNavigator(function (props) { return (React.createElement(SharedElementContext.Consumer, null, function (sharedElementRegistry) { return (React.createElement(SharedElement, __assign({}, props, { sharedElementRegistry: sharedElementRegistry }))); })); });

var SharedElementMotion = /** @class */ (function (_super) {
    __extends(SharedElementMotion, _super);
    function SharedElementMotion() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SharedElementMotion.prototype.componentDidUpdate = function (prevProps) {
        var prevSharedElements = this.getSharedElements(prevProps.sharedElements);
        var sharedElements = this.getSharedElements(this.props.sharedElements);
        this.diff(prevSharedElements, sharedElements, this.props.onAnimated);
        this.diff(sharedElements, prevSharedElements, this.props.onAnimating);
    };
    SharedElementMotion.prototype.diff = function (fromSharedElements, toSharedElements, action) {
        for (var name in fromSharedElements) {
            var from = fromSharedElements[name];
            var to = toSharedElements[name];
            if (!to || from.mountedElement.ref !== to.mountedElement.ref) {
                if (action)
                    action(name, from.oldElement.ref, from.oldElement.data);
                if (action)
                    action(name, from.mountedElement.ref, from.mountedElement.data);
            }
        }
    };
    SharedElementMotion.prototype.getSharedElements = function (sharedElements) {
        return sharedElements.reduce(function (elements, element) {
            var _a;
            return (__assign(__assign({}, elements), (_a = {}, _a[element.name] = element, _a)));
        }, {});
    };
    SharedElementMotion.prototype.getStyle = function (name, _a) {
        var ref = _a.ref, data = _a.data;
        var _b = ref.getBoundingClientRect(), top = _b.top, left = _b.left, width = _b.width, height = _b.height;
        return this.props.elementStyle(name, ref, __assign({ top: top, left: left, width: width, height: height }, data));
    };
    SharedElementMotion.prototype.getPropValue = function (prop, name) {
        return typeof prop === 'function' ? prop(name) : prop;
    };
    SharedElementMotion.prototype.render = function () {
        var _this = this;
        var _a = this.props, sharedElements = _a.sharedElements, children = _a.children, progress = _a.progress, duration = _a.duration;
        return (sharedElements.length !== 0 &&
            React.createElement(Motion, { data: sharedElements, getKey: function (_a) {
                    var name = _a.name;
                    return name;
                }, enter: function (_a) {
                    var name = _a.name, oldElement = _a.oldElement;
                    return _this.getStyle(name, oldElement);
                }, update: function (_a) {
                    var name = _a.name, mountedElement = _a.mountedElement;
                    return _this.getStyle(name, mountedElement);
                }, progress: progress < 1 ? progress : 0, duration: duration }, function (styles) { return (styles.map(function (_a) {
                var _b = _a.data, name = _b.name, oldElement = _b.oldElement, mountedElement = _b.mountedElement, style = _a.style, start = _a.start, end = _a.end;
                return (children(style, name, __assign(__assign({}, start), oldElement.data), __assign(__assign({}, end), mountedElement.data)));
            })); }));
    };
    SharedElementMotion.defaultProps = {
        duration: 300,
        elementStyle: function (name, ref, data) { return data; }
    };
    return SharedElementMotion;
}(React.Component));

var useNavigating = function (handler) {
    var navigationEvent = React.useContext(navigationReact.NavigationContext);
    React.useEffect(function () {
        var stateNavigator = navigationEvent.stateNavigator;
        var beforeNavigateHandler = function (state, data, url, history, currentContext) {
            var crumb = url.split('crumb=').length - 1;
            var _a = stateNavigator.stateContext, sceneCrumbs = _a.crumbs, sceneState = _a.state;
            if (sceneCrumbs.length === crumb && sceneState === state)
                handler(data, url, history, currentContext);
            return true;
        };
        stateNavigator.onBeforeNavigate(beforeNavigateHandler);
        return function () {
            stateNavigator.offBeforeNavigate(beforeNavigateHandler);
        };
    }, [navigationEvent, handler]);
};

var useNavigated = function (handler) {
    var navigationEvent = React.useContext(navigationReact.NavigationContext);
    React.useEffect(function () {
        handler();
    }, [navigationEvent]);
};

var useUnloading = function (handler) {
    var navigationEvent = React.useContext(navigationReact.NavigationContext);
    React.useEffect(function () {
        var stateNavigator = navigationEvent.stateNavigator;
        var beforeNavigateHandler = function (state, data, url, history, currentContext) {
            var crumb = url.split('crumb=').length - 1;
            var _a = stateNavigator.stateContext, sceneCrumbs = _a.crumbs, sceneState = _a.state;
            if (!(sceneCrumbs.length === crumb && sceneState === state)
                && stateNavigator.stateContext === currentContext) {
                var crumbs = stateNavigator.parseLink(url).crumbs;
                return handler(state, data, url, history, crumbs);
            }
            return true;
        };
        stateNavigator.onBeforeNavigate(beforeNavigateHandler);
        return function () {
            stateNavigator.offBeforeNavigate(beforeNavigateHandler);
        };
    }, [navigationEvent, handler]);
};

var useUnloaded = function (handler) {
    var navigationEvent = React.useContext(navigationReact.NavigationContext);
    React.useEffect(function () {
        var cancelled = false;
        var stateNavigator = navigationEvent.stateNavigator;
        var navigateHandler = function (_oldState, state, data, _asyncData, stateContext) {
            var crumb = stateContext.url.split('crumb=').length - 1;
            var _a = stateNavigator.stateContext, sceneCrumbs = _a.crumbs, sceneState = _a.state;
            if (!(sceneCrumbs.length === crumb && sceneState === state)
                && stateNavigator.stateContext.url === stateContext.oldUrl && !cancelled)
                handler(state, data, stateContext);
        };
        stateNavigator.onNavigate(navigateHandler);
        return function () {
            cancelled = true;
            stateNavigator.offNavigate(navigateHandler);
        };
    }, [navigationEvent, handler]);
};

var Scene$2 = NavigationMotion.Scene;

exports.MobileHistoryManager = MobileHistoryManager;
exports.NavigationMotion = NavigationMotion;
exports.Scene = Scene$2;
exports.SharedElement = SharedElement$1;
exports.SharedElementMotion = SharedElementMotion;
exports.useNavigated = useNavigated;
exports.useNavigating = useNavigating;
exports.useUnloaded = useUnloaded;
exports.useUnloading = useUnloading;
